unit QtWebBrowser;

{
  LCL Web browser control.

  Author:    Phil Hess.
  Copyright: Copyright (C) Phil Hess.
  License:   Modified LGPL (see Free Pascal's rtl/COPYING.FPC).

  Docs: http://doc.qt.io/qt-4.8/qwebview.html
}

{$mode objfpc}{$H+}

interface

uses
  SysUtils,
  LCLType,
  Controls,
  qt4,
  qtwidgets,
  WSProc,
  WebBrowser,
  WSWebBrowser;

type
  TQtWebView = class(TQtWidget)
  private
    FLoadFinishedHook : QWebView_hookH;
    FLinkClickedHook  : QWebView_hookH;
  protected
    function CreateWidget(const AParams : TCreateParams): QWidgetH; override;
  public
    procedure AttachEvents; override;
    procedure LoadFinished(LoadedOkay : Boolean); cdecl;
    procedure LinkClicked(url : QUrlH); cdecl;
  end;


  TQtWSWebBrowser = class(TWSWebBrowser)
  published
    class function CreateHandle(const AWinControl : TWinControl;
                                const AParams     : TCreateParams): TLCLIntfHandle; override;
    class procedure EnableLinkClick(const AWinControl : TWinControl;
                                          ABoolean    : Boolean); override;
    class function LoadPage(const AWinControl : TWinControl;
                            const Url         : string) : Boolean; override;
    class function LoadPageFromHtml(const AWinControl : TWinControl;
                                    const Html        : string;
                                    const RelativeUrl : string = '') : Boolean; override;
    class function EvaluateJavaScript(const AWinControl : TWinControl;
                                      const Script      : string) : string; override;
  end;


implementation

type
  TIntCustomWebBrowser = class(TCustomWebBrowser);  //so we can access protected methods


function TQtWebView.CreateWidget(const AParams : TCreateParams): QWidgetH;
var
  Parent: QWidgetH;
begin
  if AParams.WndParent <> 0 then
    Parent := TQtWidget(AParams.WndParent).GetContainerWidget
  else
    Parent := nil;
  Result := QWebView_create(Parent);  //what about QLCLWebView?
end;


procedure TQtWebView.AttachEvents;
begin
  inherited AttachEvents;
  FLoadFinishedHook := QWebView_hook_create(Widget);
  QWebView_hook_hook_loadFinished(FLoadFinishedHook, @LoadFinished);
  FLinkClickedHook := QWebView_hook_create(Widget);
  QWebView_hook_hook_linkClicked(FLinkClickedHook, @LinkClicked);
end;


procedure TQtWebView.LoadFinished(LoadedOkay : Boolean); cdecl; 
begin
  if LoadedOkay then
    TIntCustomWebBrowser(LCLObject).DoPageLoaded
  else
    TIntCustomWebBrowser(LCLObject).DoPageLoadError;
end;


procedure TQtWebView.LinkClicked(url : QUrlH); cdecl;
var
  WideUrl : WideString;
begin
  if Assigned(TIntCustomWebBrowser(LCLObject).OnLinkClick) then
    begin
    QUrl_toString(url, @WideUrl);
    TIntCustomWebBrowser(LCLObject).DoLinkClick(UTF8Encode(WideUrl));
    end;
end;



class function TQtWSWebBrowser.CreateHandle(const AWinControl : TWinControl;
                                            const AParams     : TCreateParams): TLCLIntfHandle;
var
  QtWebView: TQtWebView;
begin
  QtWebView := TQtWebView.Create(AWinControl, AParams);

  QtWebView.attachEvents;

  Result := TLCLIntfHandle(QtWebView);
end;


class procedure TQtWSWebBrowser.EnableLinkClick(const AWinControl : TWinControl;
                                                      ABoolean    : Boolean);
var
  Policy : QWebPageLinkDelegationPolicy;
begin
  if ABoolean then
    Policy := QWebPageDelegateExternalLinks
  else
    Policy := QWebPageDontDelegateLinks;
  QWebPage_setLinkDelegationPolicy(
   QWebView_page(QWebViewH(TQtWebView(AWinControl.Handle).Widget)), Policy);
end;


class function TQtWSWebBrowser.LoadPage(const AWinControl : TWinControl;
                                        const Url         : string) : Boolean;
var
  WideUrl : WideString;
  QtUrl   : QUrlH;
begin
  if not WSCheckHandleAllocated(AWinControl, 'LoadPage') then
    Exit;
  WideUrl := UTF8Decode(Url);
  QtUrl := QUrl_create(@WideUrl);
  try
    QWebView_setUrl(QWebViewH(TQtWebView(AWinControl.Handle).Widget), QtUrl);
     //note can't use QWebView_load since compiler can't determine which overloaded proc to use
//    TQtWebView(AWinControl.Handle).show;  //not needed?
  finally
    QUrl_destroy(QtUrl);
  end;
  Result := True;
end;


class function TQtWSWebBrowser.LoadPageFromHtml(const AWinControl : TWinControl;
                                                const Html        : string;
                                                const RelativeUrl : string = '') : Boolean;
var
  WideHtml : WideString;
  WideUrl  : WideString;
  QtUrl    : QUrlH;
begin
  if not WSCheckHandleAllocated(AWinControl, 'LoadPageFromHtml') then
    Exit;
  WideHtml := UTF8Decode(Html);
  WideUrl := UTF8Decode(RelativeUrl);
  QtUrl := QUrl_create(@WideUrl);
  try
    QWebView_setHtml(QWebViewH(TQtWebView(AWinControl.Handle).Widget),
                     @WideHtml, QtUrl);
  finally
    QUrl_destroy(QtUrl);
  end;
  Result := True;
end;


class function TQtWSWebBrowser.EvaluateJavaScript(const AWinControl : TWinControl;
                                                  const Script      : string) : string;
 {See http://doc.qt.io/qt-4.8/qwebframe.html#evaluateJavaScript}
var
  WideScript : WideString;
  WebPage    : QWebPageH;
  WebFrame   : QWebFrameH;
  ReturnVal  : QVariantH;
  WideResult : WideString;
begin
  if not WSCheckHandleAllocated(AWinControl, 'EvaluateJavaScript') then
    Exit;
  WideScript := UTF8Decode(Script);
  WebPage := QWebView_page(QWebViewH(TQtWebView(AWinControl.Handle).Widget));
  WebFrame := QWebPage_mainFrame(WebPage);
  ReturnVal := QVariant_create;
  try
    QWebFrame_evaluateJavaScript(WebFrame, ReturnVal, @WideScript);
    QVariant_toString(ReturnVal, @WideResult);
  finally
    QVariant_destroy(ReturnVal);
  end;
  Result := UTF8Encode(WideResult);
end;


end.
